/*Includes ----------------------------------------------*/
#include "tool.h"
#include "nrf_delay.h"
#include "bsp_time.h"
#include "exception.h"
#include "system.h"
#include "bll_imu.h"

/*Private macro ------------------------------------------------------------------------------------------------------------------------------------*/
#define							BLL_IMU_DATA_GROUP_NUM_MAX									12																			//IMUܴ洢

#define							BLL_IMU_REGISTER_CONFIG_PARAM_MAX						12																			//IMUܴ洢

#define							BLL_IMU_REGISTER_DATA_NOTIFY_MAX						5																			//IMU֪ͨص

#define 						BLL_IMU_GET_MAX(a,b) 												(((a) > (b)) ? (a) : (b))								//ԱȴС

/*STRUCTION ------------------------------------------------------------------------------------------------------------------------------------*/
typedef struct bll_imu_data_notify
{
	uint8_t 											priority;																																		//ȼ
	
	bll_imu_data_notify_cb		cb;																																					//ص
	
}Bll_Imu_Data_Notify_t;


typedef struct bll_imu
{
	/*private member*/
	const bll_imu_param_t 				*register_config_param[BLL_IMU_REGISTER_CONFIG_PARAM_MAX];						//עIMUòַ
	
	Bll_Imu_Data_Notify_t		      register_data_notify[BLL_IMU_REGISTER_DATA_NOTIFY_MAX];								//עIMU֪ͨص
	
	bll_imu_one_way_param_t 			highest_priority_config_param[BLL_IMU_DIR_NUM];												//ȼIMU

  uint8_t									config_result;																															//ý
	
} Bll_Imu_t;

/*Local Variable ------------------------------------------------------------------------------------------------------------------------------------*/
static Bll_Imu_t 									ob_bll_imu;

/*Local Functions ------------------------------------------------------------------------------------------------------------------------------------*/

static void bll_imu_register_config_cb(uint32_t conf_result)
{
	ob_bll_imu.config_result = 0;
	
	if(conf_result == FML_IMU_CONFIG_FLOW_DONE)			//óɹ
	{
		ob_bll_imu.config_result |= (2 << 6);
	}
	else																						//ʧ
	{
		ob_bll_imu.config_result = conf_result;
		ob_bll_imu.config_result |= (0 << 6);
	}

}

static void bll_imu_register_data_notify_cb(uint32_t dir_bit)
{
	//ȼʼɸѡ
	for(int i=BLL_IMU_DATA_NOTIFY_CB_PRIORITY_NUM - 1; i >= 0; i--)			
	{
		for(int j=0; j<BLL_IMU_REGISTER_DATA_NOTIFY_MAX;j++)
		{
			if(ob_bll_imu.register_data_notify[j].priority == i && ob_bll_imu.register_data_notify[j].cb != NULL)
			{
				ob_bll_imu.register_data_notify[j].cb(dir_bit);
			}
		}
	}
}

static int bll_imu_get_highest_priority_config_param(bll_imu_one_way_param_t out_param[], const bll_imu_one_way_param_t int_param[])
{
	if(out_param == NULL || int_param == NULL)return -1;
	for(uint8_t i=0; i<BLL_IMU_DIR_NUM; i++){
		out_param[i].acc_fs 								= BLL_IMU_GET_MAX(out_param[i].acc_fs, int_param[i].acc_fs);
		out_param[i].acc_odr 							  = BLL_IMU_GET_MAX(out_param[i].acc_odr, int_param[i].acc_odr);
		out_param[i].acc_power_mode 				= BLL_IMU_GET_MAX(out_param[i].acc_power_mode,int_param[i].acc_power_mode);
		out_param[i].fifo_odr 							= BLL_IMU_GET_MAX(out_param[i].fifo_odr,int_param[i].fifo_odr);
		out_param[i].gry_fs 								= BLL_IMU_GET_MAX(out_param[i].gry_fs,int_param[i].gry_fs);
		out_param[i].gry_odr 							  = BLL_IMU_GET_MAX(out_param[i].gry_odr,int_param[i].gry_odr);
		out_param[i].gry_power_mode 				= BLL_IMU_GET_MAX(out_param[i].gry_power_mode,int_param[i].gry_power_mode);
		out_param[i].mag_fs 								= BLL_IMU_GET_MAX(out_param[i].mag_fs,int_param[i].mag_fs);
		out_param[i].mag_odr 							  = BLL_IMU_GET_MAX(out_param[i].mag_odr,int_param[i].mag_odr);
		out_param[i].timestamp_resolution 	= BLL_IMU_GET_MAX(out_param[i].timestamp_resolution,int_param[i].timestamp_resolution);
		out_param[i].timestamp_switch 			= BLL_IMU_GET_MAX(out_param[i].timestamp_switch,int_param[i].timestamp_switch);
	}
	return 0;
}

/*API ------------------------------------------------------------------------------------------------------------------------------------*/
/**
	@brief 		ʼIMUҵ߼
	@param 		
	@return 						- [out]	-1ʧܣ0ɹ
*/

static const fml_imu_param_t standby_front_param={
	.acc_power_mode 						= FML_IMU_ACC_POWER_MODE_NORMAL,						//ǰ - ٶģʽ
	.gry_power_mode 						= FML_IMU_GRY_POWER_MODE_NORMAL,						//ǰ - ģʽ
	.fifo_odr 									= FML_IMU_FIFO_ODR_OFF,										  //ǰ - FIFOƵ - 104HZ
	.timestamp_resolution 			= FML_IMU_TIMESTAMP_25US,										//ǰ - ʱ25US
	.timestamp_switch 					= FML_IMU_TIMESTAMP_OFF,										//ǰ - ʱر
	.acc_odr 									  = FML_IMU_ACC_ODR_12HZ5,										//ǰ - ٶȲƵ - 12.5HZ
	.gry_odr 									  = FML_IMU_GRY_ODR_OFF,										  //ǰ - ǲƵ - ر
	.mag_odr 									  = FML_IMU_MAG_ODR_OFF,										  //ǰ - شżƲƵ - ر
	.acc_fs 										= FML_IMU_ACC_FS_16G,												//ǰ - ٶ - 16G
	.gry_fs 										= FML_IMU_GRY_FS_2000DPS,										//ǰ -  - 2000DPS
	.mag_fs 										= FML_IMU_MAG_FS_30GS,											//ǰ - شż - 30GS
	
};

static const fml_imu_param_t standby_back_param={
	.acc_power_mode 						= FML_IMU_ACC_POWER_MODE_NORMAL,						// - ٶģʽ
	.gry_power_mode 						= FML_IMU_GRY_POWER_MODE_NORMAL,						// - ģʽ
	.fifo_odr 									= FML_IMU_FIFO_ODR_OFF,											// - FIFOƵ - ر	
	.timestamp_resolution 			= FML_IMU_TIMESTAMP_25US,										// - ʱ25US
	.timestamp_switch 					= FML_IMU_TIMESTAMP_OFF,										// - ʱر
	.acc_odr 										= FML_IMU_ACC_ODR_OFF,											// - ٶȲƵ - ر
	.gry_odr 										= FML_IMU_GRY_ODR_OFF,											// - ǲƵ - ر
	.mag_odr 										= FML_IMU_MAG_ODR_10HZ,										  // - شżƲƵ - 10HZ
	.acc_fs 										= FML_IMU_ACC_FS_16G,												// - ٶ - 16G
	.gry_fs 										= FML_IMU_GRY_FS_2000DPS,										// -  - 2000DPS
	.mag_fs 										= FML_IMU_MAG_FS_30GS,											// - شż - 30GS
};

static const bll_imu_param_t standby_imu_param_t={
	.config_param[FML_IMU_DIR_FRONT] = &standby_front_param,
	.config_param[FML_IMU_DIR_BACK] =  &standby_back_param,
};

int	bll_imu_Init(void)
{
	int ret = 0;
	//ʼimu
	if(fml_imu_Init() == -1)ret = -1;
	
	//ģʽṹ
	memset(&ob_bll_imu, 0, sizeof(ob_bll_imu));
	
	//Ĭ
	ob_bll_imu.config_result |= (2 << 6);
	
	//ָȼIMU
	
	bll_imu_get_highest_priority_config_param(ob_bll_imu.highest_priority_config_param,*standby_imu_param_t.config_param);

	if(ret == -1)return -1;
	
	return 0;
}
/**
	@brief 		עIMUҪõĲ
	@param 		param							- [in]	IMUҪõĲַ
	@return 						- [out]	0ʧܣɹزַ
*/
const bll_imu_param_t* bll_imu_register_config_param(const bll_imu_param_t *param)
{
	if(param == NULL)return 0;
	
	//Ѵڣ򷵻ظָĵַΪ
	for(int i=0; i<BLL_IMU_REGISTER_CONFIG_PARAM_MAX; i++)
	{
		if(ob_bll_imu.register_config_param[i] == param)return param;
	}
	//ڣ洢ҽָĵַΪ
	for(int i=0; i<BLL_IMU_REGISTER_CONFIG_PARAM_MAX; i++)
	{
		if(ob_bll_imu.register_config_param[i] == 0)
		{
			ob_bll_imu.register_config_param[i] = param;
			
			return param;
		}
	}
	
	return 0;
}
/**
	@brief 		עעIMUò
	@param 		param							- [in]	IMUòĵַ
	@return 						- [out]	0ɹ-1ַ쳣-2ûиעIMUò
*/
int bll_imu_unregister_config_param(const bll_imu_param_t* param)
{
	if(param == NULL)return -1;
	
	for(int i=0; i<BLL_IMU_REGISTER_CONFIG_PARAM_MAX; i++)
	{
		if(ob_bll_imu.register_config_param[i] == param)
		{
			ob_bll_imu.register_config_param[i] = 0;
			return 0;
		}
	}
	
	return -2;
}
/**
	@brief 		עIMU֪ͨص
	@param 		priority 					- [in]	صȼ
	@param 		cb 								- [in]	ص
	@return 						- [out]	-1ʧܣ0ɹ
*/
int bll_imu_register_data_notify_callback(BLL_IMU_DATA_NOTIFY_CB_PRIORITY_e priority, bll_imu_data_notify_cb cb)
{
	if(cb == NULL || priority>=BLL_IMU_DATA_NOTIFY_CB_PRIORITY_NUM)return -1;
	
	//Ѵڣȼ
	for(int i=0; i<BLL_IMU_REGISTER_DATA_NOTIFY_MAX; i++)
	{
		//Ѵڻص
		if(ob_bll_imu.register_data_notify[i].cb == cb)
		{
			//ȼ
			ob_bll_imu.register_data_notify[i].priority = priority;
			return 0;
		}
	}
	//ǲ
	for(int i=0; i<BLL_IMU_REGISTER_DATA_NOTIFY_MAX; i++)
	{
		//ҵλиֵ
		if(ob_bll_imu.register_data_notify[i].cb == 0)
		{
			ob_bll_imu.register_data_notify[i].cb 				= cb;
			ob_bll_imu.register_data_notify[i].priority 	= priority;
			
			return 0;
		}
	}
	//
	return -1;
}
/**
	@brief 		ʼáעIMUɸѡȼãֻһΡ
	@param 		
	@return 						- [out]	-1ʧܣ0ɹ
*/
int bll_imu_start_config(void)
{
	int ret= 0xff;
	uint8_t i=0;
	
	//ǰȼò
  for(i=0; i<BLL_IMU_DIR_NUM; i++)
	    memset(&ob_bll_imu.highest_priority_config_param[i],0,sizeof(bll_imu_one_way_param_t));
	
	//ȡǰźͺȼ
	for(i=0; i<BLL_IMU_REGISTER_CONFIG_PARAM_MAX; i++){
		if(ob_bll_imu.register_config_param[i] != 0)
		{
		  DEBUG_LOG("!!!!!!!!!loop i \n");
			ret = bll_imu_get_highest_priority_config_param(ob_bll_imu.highest_priority_config_param, *ob_bll_imu.register_config_param[i]->config_param);
			if(ret == -1)return -1;
		}
	}
	
	if(0xff == ret){//бݣĬֵ
		bll_imu_get_highest_priority_config_param(ob_bll_imu.highest_priority_config_param,*standby_imu_param_t.config_param);
	}
	
	DEBUG_LOG("config param front :%d,%d,%d,%d,%d,%d,%d,%d,%d,%d\n",
	ob_bll_imu.highest_priority_config_param[BLL_IMU_DIR_FRONT].acc_power_mode,
	ob_bll_imu.highest_priority_config_param[BLL_IMU_DIR_FRONT].gry_power_mode,
	ob_bll_imu.highest_priority_config_param[BLL_IMU_DIR_FRONT].timestamp_resolution,
	ob_bll_imu.highest_priority_config_param[BLL_IMU_DIR_FRONT].timestamp_switch,
	ob_bll_imu.highest_priority_config_param[BLL_IMU_DIR_FRONT].acc_fs,
	ob_bll_imu.highest_priority_config_param[BLL_IMU_DIR_FRONT].gry_fs,
	ob_bll_imu.highest_priority_config_param[BLL_IMU_DIR_FRONT].mag_fs,
	ob_bll_imu.highest_priority_config_param[BLL_IMU_DIR_FRONT].acc_odr,
	ob_bll_imu.highest_priority_config_param[BLL_IMU_DIR_FRONT].gry_odr,
	ob_bll_imu.highest_priority_config_param[BLL_IMU_DIR_FRONT].mag_odr,
	ob_bll_imu.highest_priority_config_param[BLL_IMU_DIR_FRONT].fifo_odr
	);
	
  DEBUG_LOG("config param back :%d,%d,%d,%d,%d,%d,%d,%d,%d,%d\n",
	ob_bll_imu.highest_priority_config_param[BLL_IMU_DIR_BACK].acc_power_mode,
	ob_bll_imu.highest_priority_config_param[BLL_IMU_DIR_BACK].gry_power_mode,
	ob_bll_imu.highest_priority_config_param[BLL_IMU_DIR_BACK].timestamp_resolution,
	ob_bll_imu.highest_priority_config_param[BLL_IMU_DIR_BACK].timestamp_switch,
	ob_bll_imu.highest_priority_config_param[BLL_IMU_DIR_BACK].acc_fs,
	ob_bll_imu.highest_priority_config_param[BLL_IMU_DIR_BACK].gry_fs,
	ob_bll_imu.highest_priority_config_param[BLL_IMU_DIR_BACK].mag_fs,
	ob_bll_imu.highest_priority_config_param[BLL_IMU_DIR_BACK].acc_odr,
	ob_bll_imu.highest_priority_config_param[BLL_IMU_DIR_BACK].gry_odr,
	ob_bll_imu.highest_priority_config_param[BLL_IMU_DIR_BACK].mag_odr,
	ob_bll_imu.highest_priority_config_param[BLL_IMU_DIR_BACK].fifo_odr
	);
	
	//ǰҪõĲ
	ret = fml_imu_config_param((FML_IMU_DIR_e)BLL_IMU_DIR_FRONT, &ob_bll_imu.highest_priority_config_param[BLL_IMU_DIR_FRONT]);
	if(ret == -1)return -1;
	
	//úҪõĲ
	ret = fml_imu_config_param((FML_IMU_DIR_e)BLL_IMU_DIR_BACK, &ob_bll_imu.highest_priority_config_param[BLL_IMU_DIR_BACK]);
	if(ret == -1)return -1;
	
	//עûص
	ret = fml_imu_register_config_callback(bll_imu_register_config_cb);
	if(ret == -1)return -1;
	
	//ע֪ͨص
	ret = fml_imu_register_data_notify_callback(bll_imu_register_data_notify_cb);
	if(ret == -1)return -1;
	
	//ʼIMU
	ret = fml_imu_start_config();
	if(ret == 0)
	{
		//Ϊ...
		ob_bll_imu.config_result = 0;
		ob_bll_imu.config_result |= (1 << 6);
	}
	
	
	return 0;
}
/**
	@brief 		ѯIMUòǷ׼
	@param 		dir 							- [in]	
	@param 		param							- [in]	ҪѯIMUò
	@return 	ѯ					- [out] [7:6]״̬0ʧܣ1У2ɹ3ǰòСò򲻷ϣ						[0:5]ò.
*/
uint8_t bll_imu_query_config_param_is_ready(BLL_IMU_DIR_e dir, const bll_imu_param_t *param)
{
	uint8_t result = 0;
	
	if(param == NULL || dir >= BLL_IMU_DIR_NUM)
	{
		result |= (3 << 6);
		return result;
	}

	const bll_imu_one_way_param_t	*front_bll = param->config_param[BLL_IMU_DIR_FRONT];
	const bll_imu_one_way_param_t	*back_bll = param->config_param[BLL_IMU_DIR_BACK];
	
	const bll_imu_one_way_param_t	*highest_front_bll = &ob_bll_imu.highest_priority_config_param[BLL_IMU_DIR_FRONT];
	const bll_imu_one_way_param_t	*highest_back_bll = &ob_bll_imu.highest_priority_config_param[BLL_IMU_DIR_BACK];
	
	//ǰòСò
	switch(dir)
	{
		case BLL_IMU_DIR_FRONT:
			//ǰ
			if(front_bll->acc_fs > highest_front_bll->acc_fs)return result |= (3 << 6);
			if(front_bll->acc_odr > highest_front_bll->acc_odr)return result |= (3 << 6);
			if(front_bll->acc_power_mode > highest_front_bll->acc_power_mode)return result |= (3 << 6);
			if(front_bll->fifo_odr > highest_front_bll->fifo_odr)return result |= (3 << 6);
			if(front_bll->gry_fs > highest_front_bll->gry_fs)return result |= (3 << 6);
			if(front_bll->gry_odr > highest_front_bll->gry_odr)return result |= (3 << 6);
			if(front_bll->gry_power_mode > highest_front_bll->gry_power_mode)return result |= (3 << 6);
			if(front_bll->mag_fs > highest_front_bll->mag_fs)return result |= (3 << 6);
			if(front_bll->mag_odr > highest_front_bll->mag_odr)return result |= (3 << 6);
			if(front_bll->timestamp_resolution > highest_front_bll->timestamp_resolution)return result |= (3 << 6);
			if(front_bll->timestamp_switch > highest_front_bll->timestamp_switch)return result |= (3 << 6);
			break;
		
		case BLL_IMU_DIR_BACK:
			//
			if(back_bll->acc_fs > highest_back_bll->acc_fs)return result |= (3 << 6);
			if(back_bll->acc_odr > highest_back_bll->acc_odr)return result |= (3 << 6);
			if(back_bll->acc_power_mode > highest_back_bll->acc_power_mode)return result |= (3 << 6);
			if(back_bll->fifo_odr > highest_back_bll->fifo_odr)return result |= (3 << 6);
			if(back_bll->gry_fs > highest_back_bll->gry_fs)return result |= (3 << 6);
			if(back_bll->gry_odr > highest_back_bll->gry_odr)return result |= (3 << 6);
			if(back_bll->gry_power_mode > highest_back_bll->gry_power_mode)return result |= (3 << 6);
			if(back_bll->mag_fs > highest_back_bll->mag_fs)return result |= (3 << 6);
			if(back_bll->mag_odr > highest_back_bll->mag_odr)return result |= (3 << 6);
			if(back_bll->timestamp_resolution > highest_back_bll->timestamp_resolution)return result |= (3 << 6);
			if(back_bll->timestamp_switch > highest_back_bll->timestamp_switch)return result |= (3 << 6);
			break;
		
		default:
			break;
	}

	result = ob_bll_imu.config_result;
	
	return result;
}
/**
	@brief 		ȡǰIMUݵ
	@param 		dir 							- [in]	
	@return 	صǰIMUݵ
*/
int bll_imu_get_data_num(BLL_IMU_DIR_e dir)
{
	return fml_imu_get_data_num((FML_IMU_DIR_e)dir);
}
/**
	@brief 		ȡǰIMU
	@param 		dir 							- [in]	
	@param 		index 						- [in]	
	@param 		pdata 						- [out]	صIMUָ
	@return 						- [out]	-1ʧܣ0ɹ
*/
int bll_imu_get_data(BLL_IMU_DIR_e dir, int index, bll_imu_data_t *pdata)
{
	return fml_imu_get_data((FML_IMU_DIR_e)dir, index, pdata);
}
/**
	@brief 		رIMU
	@return 						- [out]	-1ʧܣ0ɹ
*/
int bll_imu_close(void)
{
	int ret;
	
	//ϵǰ
	ret = fml_imu_close(FML_IMU_DIR_FRONT);
	if(ret == -1)return -1;
	ret = fml_imu_close(FML_IMU_DIR_BACK);
	if(ret == -1)return -1;
	
	//ģʽṹ
	memset(&ob_bll_imu, 0, sizeof(ob_bll_imu));
	
	//Ĭ
	ob_bll_imu.config_result |= (2 << 6); 
	
	return 0;
}


















